function [ym,ylm,p,rls]=clfit(x,y,ct,mfs,fh)
% [ym,ylm,p,rls]=clfit(x,y,ct,mfs,fh)
% Sinus funtion fitting
% Sub-function called by FUZZCL
% x,y	Input-output data
% ct		Consequent type (1=singletons, 2=functional)
% mfs	The Membership functions
% fh	Figure handle (for plots)
%
% ym	model output
% ylm	Ylm ...	output of local submodels with data corresponding to
% 		degrees < 0.2 masked (for plots)
% p		consequent parameters
% rls	rule matrix
%
% See FUZZSINE / 

% Magne Setnes, 17/5-98

% Make RLS matrix
rls=[ones(size(mfs,1),1),[1:size(mfs,1)]'];

% Activation
dof=dofprod(rls,mfs,x);

% Parameters and approximation
if (ct==1);	% Consequents are singletons
	x_in=[];
else,
	x_in=x;
end;
[p,ym,yl,ylm] = suglms([x_in, ones(size(y,1),1)],y,dof);

%%% PLOT RESULTS %%%
% Fit of function
figure(fh);
subplot(211);
plot(x,y,'b',x,ym,'r');axis([0 100 -50 60]);
set(gca,'box','off');
xlabel('x');ylabel('y approx');

% Membership functions
subplot(212);
plot(x,mgrade(x',mfs));
xlabel('x');ylabel('Membership');
drawnow;

% Local models
figure(fh-1);
plot(x,y,'b');axis([0 100 min([ylm(find(ylm));y]) max([ylm(find(ylm));y])]);
set(gca,'box','off');hold on;
xlabel('x');ylabel('y');
plot(x,ylm,'linewidth',1.5);
hold off;





% end of function