function mfset(H,P1,V1,P2,V2,P3,V3,P4,V4,P5,V5)
% MFSET Set membership function editor properties.
%       MFSET(H,'PropertyName',PropertyValue) sets the value of the
%       specified property for the editor in figure with handle H.
%       H can be a vector of figure handles, in which case MFSET sets
%       the editor properties for all specified figures.
%
%       MFSET(H,'PropertyName1',PropertyValue1,'PropertyName2',PropertyValue2,...)
%	sets multiple property values with a single statement.
%
% Available properties:
%       Position - window position (e.g. [100 100 200 200])
%       Input    - crisp input     (e.g. 0.5)
%       Mfs      - membership function(s) definition matrix
%                  (e.g. [2 -0.3 0 0 0.3] for bell-shaped function
%                  with kernel in 0 and support (-0.3,0.3), vector of
%                  membership function types, or just a number
%                  of membership functions
%       Domain   - input domain (universe of discourse) string
%                  (e.g. '-10:0.1:10')
%       Cmf      - current membership function
%
% Property names are case insensitive.
%
% Editor usage: use sliders to change the shape or to move MF's.
%               MF can be selected and deselected by clicking its curve. MF
%               type popup options and sliders apply to the selected MF or
%               to all functions if no function is selected.
%
% See also: MFGET

% (c) Robert Babuska, August 1993

% Initialization
nprop = (nargin - 1)/2;         % number of property - value pairs
for i = 1 : nprop,              % convert property names to lower-case
  str = num2str(i);
  eval(['P' str ' = lower(P' str ');']);
end;

figs = get(0,'Children');
if length(figs) == 0; figs = 0; end;
fighere = 0;

if (nargin <  1)
  H = max(figs) + 1;
end;

for fig = 1 : length(H),
cf = H(fig);
if any(cf == figs),
  if (length(get(cf,'Children')) == 8), fighere = 1; end;
end;

% Handle callbacks
if nprop == 1
  property = P1; value = V1;
  if strcmp(property,'callback')

   if fighere,
    % Get handles and other data from UserData
    hc = get(H,'UserData');
    cmf = hc(length(hc));
    x = get(hc(6),'UserData'); rx = max(x) - min(x);
    mf = get(get(H,'CurrentAxes'),'UserData');
%    x1 = eval(get(hc(6),'String'))';
%    for cmf = 1 : size(mf,1),
%       ind = find(mf(cmf,2:5) > max(x1));
%       if ~isempty(ind), mf(cmf,ind+1) = max(x1); end;
%       ind = find(mf(cmf,2:5) < min(x1));
%       if ~isempty(ind), mf(cmf,ind+1) = min(x1); end;
%    end;   
    if length(mf) == 0, return; end;
    mfn = length(mf(:,1));
    if isempty(mf), error('MF''s are not in the plot.'); end;
    P = mf(:,6);

    % Callbacks
    if (value == 1)            % MF sliders callback
      co = find(hc(1:4) == get(H,'CurrentObject'));
      if (cmf ~= 0),
	mf(cmf,co+1) = get(hc(co),'Value');
        switch co+1,
        case 2,
          if mf(cmf,co+1) > mf(cmf,3), mf(cmf,co+1) = mf(cmf,3); end;
        case 3,
          if mf(cmf,co+1) > mf(cmf,4), mf(cmf,co+1) = mf(cmf,4); end;
          if mf(cmf,co+1) < mf(cmf,2), mf(cmf,co+1) = mf(cmf,2); end;
        case 4,
          if mf(cmf,co+1) > mf(cmf,5), mf(cmf,co+1) = mf(cmf,5); end;
          if mf(cmf,co+1) < mf(cmf,3), mf(cmf,co+1) = mf(cmf,3); end;
        case 5,
          if mf(cmf,co+1) < mf(cmf,4), mf(cmf,co+1) = mf(cmf,4); end;
        end;
        set(gca, 'UserData',mf);
        set(P(cmf), 'ydata', mgrade(x',mf(cmf,1:5))');
      else
        val = get(hc(co),'Value');
        dval = val - get(hc(co),'UserData');
        set(hc(co),'UserData',val);
        co = co + 1;
        if co == 2 | co == 3,
          mf(mfn,co) = mf(mfn,co) + dval;
        end;
        if co == 4 | co == 5,
          mf(1,co) = mf(1,co) + dval;
        end;
        for i = 2 : mfn-1,
          mf(i,co) = mf(i,co) + dval;
        end;
        for i = 1 : mfn,
          if co == 2, if mf(i,2) > mf(i,3), mf(i,2) = mf(i,3); end; end;
          if co == 5, if mf(i,5) < mf(i,4), mf(i,5) = mf(i,4); end; end;
          if co == 3,
            if mf(i,3) > mf(i,4), mf(i,3) = mf(i,4); end;
            if mf(i,3) < mf(i,2), mf(i,3) = mf(i,2); end;
          end;
          if co == 4,
            if mf(i,4) > mf(i,5), mf(i,4) = mf(i,5); end;
            if mf(i,4) < mf(i,3), mf(i,4) = mf(i,3); end;
          end;
          set(P(i), 'ydata', mgrade(x',mf(i,1:5))');
        end;
        set(gca, 'UserData',mf);
      end;

    elseif (value == 2)        % Popup callback
      mft = get(hc(5),'Value');
      x = get(hc(6),'UserData'); rx = max(x) - min(x);
      if (cmf == 0),
	for i = 1 : mfn,
          mf(i,1) = mft;
          set(P(i), 'ydata', mgrade(x',mf(i,1:5))');
	end;
      else,
        mf(cmf,1) = mft;
        set(P(cmf), 'ydata', mgrade(x',mf(cmf,1:5))');
      end;
      set(gca, 'UserData',mf);

    elseif (value == 3)        % Domain edit callback
      x1 = eval(get(hc(6),'String'))';
      r = rx/(max(x1)-min(x1));
      rx = max(x1)-min(x1);
      set(hc(6),'UserData',x1);
      axis([min(x1) max(x1) 0 1]);
      mf(:,2:5) = mf(:,2:5)./r;
      for i = 1 : size(mf,1),
         ind = find(mf(i,2:5) > max(x1));
         if ~isempty(ind), mf(i,ind+1) = max(x1); end;
         ind = find(mf(i,2:5) < min(x1));
         if ~isempty(ind), mf(i,ind+1) = min(x1); end;
      end;   
      set(gca, 'UserData',mf);
      for i = 1 : mfn,
        set(P(i), 'xdata',x1, 'ydata', mgrade(x1',mf(i,1:5))');
      end;
      aux = get(hc(7),'UserData');
      set(hc(7),'Min',min(x1)-eps,'Max',max(x1)+eps,'UserData',[aux(1) aux(2)/r aux(2)/r]);
      set(aux(1),'xdata',[aux(2)/r aux(2)/r]);
      if cmf == 0,
         val = aux(2)/r;
         if val > max(x1), val = max(x1); end;
         if val < min(x1), val = min(x1); end;
         set(hc(7),'Value',val);
      else
        set(hc(7),'Value',(mf(cmf,3)+mf(cmf,4))/2);
      end;

      if cmf == 0,
        set(hc(1),'Min',-rx-eps,'Max',0+eps,'UserData',-rx/2,'Value',-rx/2);
        set(hc(2),'Min',-rx-eps,'Max',0+eps,'UserData',-rx/2,'Value',-rx/2);
        set(hc(3),'Min',0-eps,'Max', rx+eps,'UserData',rx/2,'Value',rx/2);
        set(hc(4),'Min',0-eps,'Max', rx+eps,'UserData',rx/2,'Value',rx/2);
        aux = get(hc(7),'UserData');
        aux(2) = min(max(aux(2),get(hc(7),'Min')),get(hc(7),'Max'));
        set(hc(7),'Value',aux(2));
      else
        x1 = eval(get(hc(6),'String'))';
        ind = find(mf(cmf,2:5) > max(x1));
        if ~isempty(ind), mf(cmf,ind+1) = max(x1); end;
        ind = find(mf(cmf,2:5) < min(x1));
        if ~isempty(ind), mf(cmf,ind+1) = min(x1); end;
        set(hc(1),'Min',min(x1)-eps,  'Max',mf(cmf,3)+eps,'Value',mf(cmf,2));
        set(hc(2),'Min',mf(cmf,2)-eps,'Max',mf(cmf,4)+eps,'Value',mf(cmf,3));
        set(hc(3),'Min',mf(cmf,3)-eps,'Max',mf(cmf,5)+eps,'Value',mf(cmf,4));
        set(hc(4),'Min',mf(cmf,4)-eps,'Max',max(x1)+eps,  'Value',mf(cmf,5));
        aux = get(hc(7),'UserData');
        set(hc(7),'Value',(mf(cmf,3)+mf(cmf,4))/2, ...
                  'UserData',[aux(1:2) (mf(cmf,3)+mf(cmf,4))/2]);
      end;
    elseif (value == 4)        % Value or shift slider callback
      hc = get(H,'UserData');
      input = get(hc(7),'Value');
      if cmf == 0,
        aux = get(hc(7),'UserData');
        set(aux(1),'xdata',[input input]);
        set(hc(7),'UserData',[aux(1) input input]);
      else
        val = get(hc(7),'Value');
        aux = get(hc(7),'UserData');
        dval = val - aux(3);
        set(hc(7),'UserData',[aux(1:2) val]);
        mf(cmf,2:5) = mf(cmf,2:5)+dval;
%        x1 = eval(get(hc(6),'String'))';
%        ind = find(mf(cmf,2:5) > max(x1));
%        if ~isempty(ind), mf(cmf,ind+1) = max(x1); end;
%        ind = find(mf(cmf,2:5) < min(x1));
%        if ~isempty(ind), mf(cmf,ind+1) = min(x1); end;
        set(P(cmf), 'ydata', mgrade(x',mf(cmf,1:5))');
        set(gca, 'UserData',mf);
      end;
    elseif (value == 5)        % Membership function selection callback
        hm = mf(1:mfn,6);
        co = [];
        if ~isempty(get(H,'CurrentObject')),
           co = find(hm == get(H,'CurrentObject'));
        end;   
        if (length(co) == 1),
          if (cmf ~= 0),
            set(hm(cmf),'Color','b','LineWidth',0.5);
            aux = get(hc(7),'UserData');
            aux(2) = min(max(aux(2),get(hc(7),'Min')),get(hc(7),'Max'));
            set(hc(7),'Value',aux(2));
          end;
          if (cmf == co),
            hc(length(hc)) = 0;
          elseif (cmf ~= co),
	    hc(length(hc)) = co;
	    set(hm(co),'Color','g','LineWidth',2);
	    cmf = co;
            aux = get(hc(7),'UserData');
            set(hc(7),'UserData',[aux(1:2) (mf(cmf,3)+mf(cmf,4))/2]);
	  end;
          set(H,'UserData',hc);
          set(hc(5),'Value',mf(cmf,1));
          cmf = hc(length(hc));
        end;

    % Set slider ranges and current values

    end;
    if cmf == 0,
        set(hc(1),'Min',-rx-eps,'Max',0+eps,'Value',get(hc(1),'UserData'));
        set(hc(2),'Min',-rx-eps,'Max',0+eps,'Value',get(hc(2),'UserData'));
        set(hc(3),'Min',0-eps,'Max', rx+eps,'Value',get(hc(3),'UserData'));
        set(hc(4),'Min',0-eps,'Max', rx+eps,'Value',get(hc(4),'UserData'));
        aux = get(hc(7),'UserData');
        aux(2) = min(max(aux(2),get(hc(7),'Min')),get(hc(7),'Max'));
        set(hc(7),'Value',aux(2));
    else
        x1 = eval(get(hc(6),'String'))';
        ind = find(mf(cmf,2:5) > max(x1));
        if ~isempty(ind), mf(cmf,ind+1) = max(x1); end;
        ind = find(mf(cmf,2:5) < min(x1));
        if ~isempty(ind), mf(cmf,ind+1) = min(x1); end;
        set(hc(1),'Min',min(x)-eps,   'Max',mf(cmf,3)+eps,'Value',mf(cmf,2));
        set(hc(2),'Min',mf(cmf,2)-eps,'Max',mf(cmf,4)+eps,'Value',mf(cmf,3));
        set(hc(3),'Min',mf(cmf,3)-eps,'Max',mf(cmf,5)+eps,'Value',mf(cmf,4));
        set(hc(4),'Min',mf(cmf,4)-eps,'Max',max(x)+eps,   'Value',mf(cmf,5));
        set(hc(7),'Value',(mf(cmf,3)+mf(cmf,4))/2);
    end;
  end;
  return
  end;
end;

% handle user-supplied information

% editor not initialized

if ~fighere

  % initialize with defaults
  mfn = 5; mft = 1; input = 0;
  cmf = 0; 

  % scan user-supplied properties
  for i = 1 : nprop,
    str = num2str(i);
    eval(['property = P' str ';']);
    eval(['value = V' str ';']);
    if strcmp(property,'position'),
      position = value;
    elseif strcmp(property,'mfs'),
      sv = size(value);
      if max(sv) == 1,
        mfn = value; mft = 1;
      elseif sv(2) == 1,
        mfn = length(value);
        mft = value;
      else
        mf = value;
      end;
    elseif strcmp(property,'domain'),
      dom = value;
    elseif strcmp(property,'input'),
      input = value;
    elseif strcmp(property,'cmf'),
      cmf = value;
    end;
  end;

  % set all properties

  if ~any(cf == figs),
    if exist('position') ~= 1, cf = figure('visible','off');
    else cf = figure('visible','off','position',position); end;
  else
    figure(cf);
  end;
  if (exist('dom')~=1),
    if (exist('mf')~=1)
      dom = '-1:0.001:1';
    else
      x = [min(mf(:,2)) max(mf(:,5))];
      dom = [num2str(x(1)) ':' ... 
	     num2str((x(2) - x(1))/40) ':' ...
	     num2str(x(2))];
    end;
  end;
  x = eval(dom)';
  rx = max(x) - min(x);
  if (exist('mf')~=1)
    if (mfn > 1),
      mf = mfequ(x',mft,mfn);
    else,
      mf = [1 min(x) min(x)+rx/2 min(x)+rx/2 max(x)];
    end;
  end;
  y = mgrade(x',mf)';
  clf;
  axes('Position',[0.13 0.2 0.77 0.6]);
%  P = plot(x,y,[input input],[0 1],'EraseMode','xor','Color','b');
  P = plot(x,y,[input input],[0 1],'Color','b');
  Pl = length(P);
  set(P(Pl),'Color','r','LineWidth',2);
  if (cmf ~= 0),
    set(P(cmf),'Color','g','LineWidth',2);
  end;
  axis([min(x) max(x) 0 1]);

  % Define user controls
  hc = [ uicontrol('Style','slider','Units','normalized','Position',[0.02 0.01 0.30 0.05],...
          'CallBack','mfset(gcf,''CallBack'',1);'); ...

	 uicontrol('Style','slider','Units','normalized','Position',[0.02 0.94 0.30 0.05],...
          'CallBack','mfset(gcf,''CallBack'',1);'); ...

	 uicontrol('Style','slider','Units','normalized','Position',[0.66 0.94 0.30 0.05],...
          'CallBack','mfset(gcf,''CallBack'',1);'); ...

	 uicontrol('Style','slider','Units','normalized','Position',[0.66 0.01 0.30 0.05],...
          'CallBack','mfset(gcf,''CallBack'',1);'); ...

	 uicontrol('Style','Popup', 'String',...
	 'Trapezoidal|Exponential|Sigmoidal','Units','normalized',...
         'Position',[0.37 0.92 0.26 0.06],'CallBack','mfset(gcf,''CallBack'',2);'); ...

         uicontrol('Style','Edit','Units','normalized','Position',[0.37 0.01 0.26 0.06],...
          'CallBack','mfset(gcf,''CallBack'',3);',...
	  'String',dom);...

	 uicontrol('Style','slider','Units','normalized','Position',[0.08 0.83 0.87 0.04],...
	  'Min',min(x)-eps,'Max',max(x)+eps,...
          'CallBack','mfset(gcf,''CallBack'',4);')];

         set(hc(1),'Min',-rx-eps,'Max',0+eps,'Value',-rx/2,'UserData',-rx/2);
         set(hc(2),'Min',-rx-eps,'Max',0+eps,'Value',-rx/2,'UserData',-rx/2);
         set(hc(3),'Min',0-eps,'Max', rx+eps,'Value',rx/2,'UserData',rx/2);
         set(hc(4),'Min',0-eps,'Max', rx+eps,'Value',rx/2,'UserData',rx/2);

  set(cf,'WindowButtonDownFcn','mfset(gcf,''CallBack'',5);');

  set(cf, 'UserData',[hc; cmf]);
  set(hc(5),'Value',mf(1,1));
  set(hc(6),'UserData',x);
  x1 = eval(get(hc(6),'String'))';
  set(hc(7),'UserData',[P(Pl) input input]);
  if cmf == 0,
    aux = get(hc(7),'UserData');
    aux(2) = min(max(aux(2),get(hc(7),'Min')),get(hc(7),'Max'));
    set(hc(7),'Value',aux(2));
  else
    set(hc(7),'Value',(mf(cmf,3)+mf(cmf,4))/2);
         end;
  P = P(1:Pl-1);
  mf = [mf P];
  set(gca,'UserData',mf);
  mfset(gcf,'CallBack',5);
  set(cf,'visible','on');

elseif fighere

  hc = get(cf,'UserData');

  % Scan user-supplied properties
  for i = 1 : nprop,
    x = get(hc(6),'UserData'); rx = max(x) - min(x);
    input = get(hc(7),'value');
    str = num2str(i);
    eval(['property = P' str ';']);
    eval(['value = V' str ';']);

    if strcmp(property,'position'),
      set(cf,'Position',value);
    elseif strcmp(property,'mfs'),
      domain = get(hc(6),'String');
      input = get(hc(7),'Value');
      figure(cf); clf; mfset(cf,'domain',domain,'input',input,'mfs',value);
    elseif strcmp(property,'domain'),
      figure(cf); set(hc(6),'String',value); mfset(cf,'CallBack',3);
    elseif strcmp(property,'input'),
      value = min(max(value,get(hc(7),'Min')),get(hc(7),'Max'));
      set(hc(7),'Value',value); mfset(cf,'CallBack',4);
    elseif strcmp(property,'cmf')
      P = get(gca,'UserData'); P = P(:,6);
      cmf = hc(length(hc));
      if cmf ~= 0, set(P(cmf),'Color','b','LineWidth',0.5); end;
      if value~=0, set(P(value),'Color','g','LineWidth',2); end;
      hc(length(hc)) = value; set(cf,'UserData',hc);
    end;
  end;
end;
end;
