% Introduction to PRESSURE exercise
%
% Called by fuzzPRES

%--------------------------
% Instructions window
%--------------------------
figure(instr);
set(gca,'position',[0 0 1 1]);
axis off;
htext = text(0.05,0.9,'FUZZY MODELING of PRESSURE DYNAMICS');
htext = text(0.05,0.7,'In an experiment, the fermenter is exited by a');
htext = text(0.05,0.6,'constant air in-flow at the bottom, while the');
htext = text(0.05,0.5,'air-outlet valve at the top is varied.');
htext = text(0.05,0.4,'Both the pressure and the valve position are');
htext = text(0.05,0.3,'sampled in order to collect identification data.');
htext = text(0.05,0.1,'Press [ENTER] to see the identification data.');
set(instr,'Name','Introduction: Pressure Dynamics',...
	'Visible','on');

%--------------------------
% Open illustartion window
%--------------------------
figure(illustr);
set(gca,'position',[0 0 1 1]);
axis off;
image(x); colormap(map); 
set(illustr,'Name','Fermenter',...
	'Visible','on');

pause on;pause;
% Close illustration and delete photo from workspace
figure(illustr);clf;
clear map x;
colormap('default');

%------------------------
% The identification data
%------------------------
pos=urightbig;
set(illustr,'Name','Identification data','Position',pos);
subplot(211);
tijd=0:5:5*(size(trd,1)-1);
plot(tijd,trd(:,1),'b');
axis([min(tijd) max(tijd) .95 max(trd(:,1))]);
set(gca,'box','off');
xlabel('Time [sec]');
ylabel('Pressure y [10e5 Pa]');
subplot(212);
plot(tijd,trd(:,2),'b');
axis([min(tijd) max(tijd) 0 max(trd(:,2))]);
set(gca,'box','off');
xlabel('Time [sec]');
ylabel('Valve u [% closed]');

% Change text in instruction window
figure(instr);clf;
set(gca,'position',[0 0 1 1]);
axis off;
htext = text(0.05,0.9,'IDENTIFICATION DATA');
htext = text(0.05,0.7,'The fermentor pressure as a function of time');
htext = text(0.05,0.6,'and the applied control (valve position) are');
htext = text(0.05,0.5,'both shown on the right.');
htext = text(0.05,0.2,'Press [ENTER] to see model structure.');
pause on;pause;

%-----------------
% MODEL STRUCTURE
%-----------------
figure(instr);clf;
set(gca,'position',[0 0 1 1]);
axis off;
htext = text(0.05,0.9,'MODEL STRUCTURE');
htext = text(0.05,0.7,'Pressure at time (k+1) is modeled as a function');
htext = text(0.05,0.6,'of the pressure and the valve position at time (k):');
htext = text(0.05,0.5,'y(k+1) = f( y(k), u(k) )');
htext = text(0.05,0.4,'The used identification data (u(k),y(k)) is shown to the right.');
htext = text(0.05,0.2,'Press [ENTER] to continue.');

% Plot identification data
figure(illustr);clf;
pos=uright;
set(illustr,'Name','Identification data','Position',pos,'visible','on');
plot(trd(:,2),trd(:,1),'ob','markersize',3);
xlabel('Valve u(k) [% closed]');
ylabel('Pressure y(k) [10e5 Pa]');

pause on;pause;

%---------------------
% Modeling instruction
%---------------------
figure(instr);clf;
set(gca,'position',[0 0 1 1]);
axis off;
htext = text(0.05,0.9,'MODELING');
htext = text(0.05,0.7,'Start by defining a fuzzy grid in the input space.');
htext = text(0.05,0.6,'Select the number of fuzzy sets to use for');
htext = text(0.05,0.5,'partitioning the two inputs u(k) and y(k).');
htext = text(0.05,0.4,'Define your model, edit it and simulate...');
htext = text(0.05,0.2,'Press [ENTER] to start modeling.');

pause on;pause;
set(instr,'visible','off');

% End of introduction