function [ym,ylm,p,rls]=sinefit(x,y,ct,mfs,fh)
% [ym,ylm,p,rls]=sinefit(x,y,ct,mfs,fh)
% Sinus funtion fitting
% Sub-function called by fuzzSINE
% x,y	Input-output data
% ct		Consequent type (1=singletons, 2=functional)
% mfs	The Membership functions
% fh	Figure handle (for plots)
%
% ym	model output
% ylm	Ylm ...	output of local submodels with data corresponding to
% 		degrees < 0.2 masked (for plots)
% p		consequent parameters
% rls	rule matrix
%
% See fuzzSINE

% Magne Setnes, 17/5-98

% Make RLS matrix
rls=[ones(size(mfs,1),1),[1:size(mfs,1)]'];

% Activation
dof=dofprod(rls,mfs,x);

% Parameters and approximation
if (ct==1);	% Consequents are singletons
	x_in=[];
else,
	x_in=x;
end;
[p,ym,yl,ylm] = suglms([x_in, ones(size(y,1),1)],y,dof);

%%% PLOT RESULTS %%%
% Local models
figure(fh);
xlabel('x');ylabel('y');
plot(x,ylm,'k-','linewidth',1.5);
axis([0 100 min([ylm(find(ylm));y]) max([ylm(find(ylm));y])]);


% Fit of function
hold on
plot(x,y,'b-',x,ym,'r-');axis([0 100 -60 60]);
set(gca,'xtick',[0:20:100]');
set(gca,'box','off');
set(gca,'xgrid','on');
xlabel('x');ylabel('y');
hold off;

% end of function